/* eslint-disable react-native/no-inline-styles */
import {FlatList, PanResponder, StyleSheet, View} from 'react-native';
import React, {createRef, useEffect, useRef, useState} from 'react';
import Circle from '../Circle';
import Scale from '../../../transitions/Scale';
import Fade from '../../../transitions/Fade';
import {Button} from 'react-native-paper';

const slotsContainerHeightCalculator = (slotHeight, margin) => {
  return 9 * slotHeight + 7 * margin;
};

const offsetToPositionCenter = (slotHeight, margin) => {
  return slotHeight * 4 + 5 * margin;
};

const scaleFactor = (activeIndex, currentIndex) => {
  if (activeIndex - currentIndex === 0) {
    return 1;
  } else if (Math.abs(activeIndex - currentIndex) < 5) {
    return 1 - 0.1 * Math.abs(activeIndex - currentIndex);
  } else {
    return 0.5;
  }
};

const scaleFactorForFreeFlow = (activeIndex, currentIndex) => {
  if (activeIndex === currentIndex) {
    return 1;
  }
  return 0.5;
};

const opacityFactor = (activeIndex, currentIndex) => {
  if (activeIndex - currentIndex === 0) {
    return 1;
  } else if (Math.abs(activeIndex - currentIndex) < 5) {
    return 1 - 0.2 * Math.abs(activeIndex - currentIndex);
  } else {
    return 0.2;
  }
};

const opacityFactorForFreeflow = (activeIndex, currentIndex) => {
  if (activeIndex === currentIndex) {
    return 1;
  }
  return 0.5;
};

const SlotList = props => {
  const slotItemHeight = 30;
  const slotItemSpacing = 4;

  const {slots, activeSlot, activeSlotIndex, setActiveSlot} = props;
  const freeflow = Boolean(props.freeflow);

  // to record and set the list container height
  // used for calculating initial and final item position offsets
  const [containerHeight, setContainerHeight] = useState(0);

  // listView ref used for triggering declarative scroll actions
  const ref = useRef(null);

  // contains the actual gesture value ("right" | "left")
  // with a timestamp applied to allow for re-setting of same
  // type of gesture
  const [gesture, setGesture] = useState('');

  // contains width value for each button, (since due to different word length)
  // center position via offset requires unique calculation
  // for each button
  const [buttonWidths, setButtonWidths] = useState([]);

  // swipe sensitivity used, this is used to trigger swipe
  // only after the finger has moved across a certain threshold of distance
  const sensitivity = 30;

  // contains animation handle refs for each animated Component wrapper on list item
  // to allow for transitions to be triggered declaratively
  const [scaleRefs, setScaleRefs] = useState([]);
  const [fadeRefs, setFadeRefs] = useState([]);

  const [scrollActivity, setScrollActivity] = useState(true);

  const panResponder = useRef(
    PanResponder.create({
      onMoveShouldSetPanResponder: () => !freeflow,
      onMoveShouldSetPanResponderCapture: () => !freeflow,
      onPanResponderRelease: (_, gestureState) => {
        if (!freeflow) {
          // used to calculate if the user has swiped up/down
          if (gestureState.dy > sensitivity) {
            setGesture('down%SEPARATE%' + Date.now().toString());
          } else if (gestureState.dy < -sensitivity) {
            setGesture('up%SEPARATE%' + Date.now().toString());
          }
        }
      },
      onPanResponderTerminationRequest: () => !freeflow,
    }),
  ).current;

  // used to set the ref handle placeholders
  // for animation on each item to be triggered on component mount
  useEffect(() => {
    if (!scaleRefs.find(d => (d.current ? true : false)))
      setScaleRefs([
        ...Array(slots.length)
          .fill(null)
          .map((_, __) => createRef()),
      ]);
    if (!fadeRefs.find(d => (d.current ? true : false)))
      setFadeRefs([
        ...Array(slots.length)
          .fill(null)
          .map((_, __) => createRef()),
      ]);
    setButtonWidths([
      ...Array(slots.length)
        .fill(null)
        .map((_, __) => 0),
    ]);
  }, [slots]);

  // used to run a effect, based on the gesture user has selected
  useEffect(() => {
    const g = gesture.split('%SEPARATE%')[0];
    // edge case to allow up swipe to move to next slot only if its not the last slot in state list
    // edge case to allow down swipe to move to first slot only if its not the first slot in state list
    if (g === 'up') {
      setActiveSlot(activeSlotIndex + 1);
    } else if (g === 'down') {
      setActiveSlot(activeSlotIndex - 1);
    }
    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [gesture]);

  // effect to display on the UI which slot is active and in center
  useEffect(() => {
    setImmediate(() => {
      if (ref.current) {
        if (containerHeight > 0 && slots.length > 0 && activeSlot) {
          // makes the listView declaratively scroll onto the given index
          // with a calculated offset
          ref.current.scrollToIndex({
            index: activeSlotIndex,
            viewOffset: offsetToPositionCenter(slotItemHeight, slotItemSpacing),
          });
        } else if (activeSlotIndex === 0) {
          ref.current.scrollToIndex({
            index: activeSlotIndex,
            viewOffset: offsetToPositionCenter(slotItemHeight, slotItemSpacing),
          });
        }
        // triggers animation on each slot declaratively based on its index
        scaleRefs.forEach((scaleRef, i) => {
          if (scaleRef.current) {
            scaleRef.current.triggerAnimation(
              freeflow
                ? scaleFactorForFreeFlow(activeSlotIndex, i)
                : scaleFactor(activeSlotIndex, i),
            );
          }
        });
        fadeRefs.forEach((fadeRef, i) => {
          if (fadeRef.current) {
            fadeRef.current.triggerAnimation(
              freeflow
                ? opacityFactorForFreeflow(activeSlotIndex, i)
                : opacityFactor(activeSlotIndex, i),
            );
          }
        });
      }
    });

    // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [activeSlot]);

  const defineSlotListItemColor = slot => {
    if (slot.isUploaded) {
      return 'rgba(38, 49, 147, 0.6)';
    } else if (slot.isComplete) {
      return 'rgba(38, 147, 49, 0.6)';
    } else {
      return 'white';
    }
  };

  useEffect(() => {
    let timeoutCounter;
    if (scrollActivity) {
      timeoutCounter = setTimeout(() => setScrollActivity(false), 1000);
    }
    return () => {
      clearTimeout(timeoutCounter);
    };
  }, [scrollActivity]);

  return (
    <View
      {...panResponder.panHandlers}
      // used to record width of container in which this slot list resides
      onLayout={e => setContainerHeight(e.nativeEvent.layout.height)}
      style={{
        ...styles.root,
        height: slotsContainerHeightCalculator(slotItemHeight, slotItemSpacing),
      }}>
      <FlatList
        ref={ref}
        showsHorizontalScrollIndicator={false}
        showsVerticalScrollIndicator={false}
        data={slots}
        keyExtractor={item => item.slotId}
        scrollEnabled={freeflow}
        onScroll={_ => setScrollActivity(true)}
        onScrollToIndexFailed={() => console.log('scroll error')}
        nestedScrollEnabled={false}
        contentContainerStyle={styles.list}
        onLayout={() => {
          setImmediate(() => {
            ref.current?.scrollToIndex({
              index: 0,
              viewOffset: offsetToPositionCenter(
                slotItemHeight,
                slotItemSpacing,
              ),
            });
          });
        }}
        renderItem={data => {
          return (
            <View
              key={data.item.title}
              style={{
                ...styles.buttonContent,
                height: 50,
                // sets a initial offset from top or bottom based on position of element
                marginBottom:
                  data.index === slots.length - 1
                    ? offsetToPositionCenter(slotItemHeight, slotItemSpacing)
                    : slotItemSpacing,
                marginTop:
                  data.index === 0
                    ? offsetToPositionCenter(slotItemHeight, slotItemSpacing)
                    : 0,
              }}>
              <Scale
                ref={scaleRefs[data.index]}
                config={{
                  duration: 1000,
                  startScale: 0,
                  endScale: freeflow
                    ? scaleFactorForFreeFlow(activeSlotIndex, data.index)
                    : scaleFactor(activeSlotIndex, data.index),
                }}>
                <Fade
                  ref={fadeRefs[data.index]}
                  config={{
                    duration: 1000,
                    startOpacity: 0,
                    endOpacity: freeflow
                      ? data.item.slotId === activeSlot?.slotId
                        ? 1
                        : 0.5
                      : opacityFactor(activeSlotIndex, data.index),
                  }}>
                  <Circle
                    style={{
                      marginRight: 3,
                    }}
                    radius={slotItemHeight - 10}
                    // shows the element color based on its progress and active position
                    backgroundColor={defineSlotListItemColor(data.item)}
                  />
                </Fade>
              </Scale>
              <Button
                onLayout={e => {
                  const widthState = buttonWidths;
                  widthState[data.index] = e.nativeEvent.layout.width;
                  setButtonWidths([...widthState]);
                }}
                onPress={() => {
                  if (freeflow) {
                    setActiveSlot(data.index);
                  }
                }}
                style={{
                  opacity: freeflow
                    ? 1
                    : data.item.slotId === activeSlot.slotId
                    ? 1
                    : 0,
                  backgroundColor: freeflow
                    ? data.item.slotId === activeSlot.slotId
                      ? 'rgba(0, 0, 0, 0.5)'
                      : 'transparent'
                    : 'rgba(0, 0, 0, 0.5)',
                }}
                textColor={
                  freeflow
                    ? data.item.slotId === activeSlot.slotId
                      ? 'white'
                      : scrollActivity
                      ? 'white'
                      : 'rgba(0, 0, 0, 0.1)'
                    : 'white'
                }>
                {data.item.title}
              </Button>
            </View>
          );
        }}
      />
    </View>
  );
};

export default React.memo(SlotList);

const styles = StyleSheet.create({
  root: {
    zIndex: 3,
    position: 'absolute',
    left: 5,
    alignSelf: 'center',
    top: '30%',
  },
  buttonContent: {
    flexDirection: 'row',
    alignItems: 'center',
    justifyContent: 'space-between',
  },
  list: {
    alignItems: 'flex-start',
    justifyContent: 'center',
  },
});
