import React, {forwardRef, useImperativeHandle, useState} from 'react';
import {
  StyleSheet,
  Text,
  TextStyle,
  TouchableWithoutFeedback,
  View,
  ViewStyle,
} from 'react-native';
import {
  RadioButton,
  RadioButtonProps,
  TextInput,
  TextInputProps,
} from 'react-native-paper';

type FormFieldProps = Omit<
  TextInputProps & {initialValue: string},
  'value' | 'onChangeText' | 'ref'
>;

export type FormFieldRef = {
  get: () => string | boolean;
};

export const FormField = forwardRef<FormFieldRef, FormFieldProps>(
  (props, ref) => {
    const [input, setInput] = useState(
      props.initialValue ? props.initialValue : '',
    );
    useImperativeHandle(ref, () => ({
      get() {
        return input;
      },
    }));
    return (
      <TextInput
        {...props}
        value={input}
        onChangeText={text => setInput(text)}
      />
    );
  },
);

type RadioGroupProps = Omit<
  {
    options: string[];
    label: string;
    viewStyle?: ViewStyle;
    labelStyle?: TextStyle;
    initialValue?: string;
  } & RadioButtonProps,
  'status' | 'onPress' | 'value' | 'style'
>;
type RadioGroupRef = {
  get: () => string | boolean;
};

export const RadioGroup = forwardRef<RadioGroupRef, RadioGroupProps>(
  (props, ref) => {
    const [checked, setChecked] = useState(
      props.initialValue ? props.initialValue : '',
    );

    const onPressHandler = (option: string) =>
      checked === option ? setChecked('') : setChecked(option);

    useImperativeHandle(ref, () => ({
      get() {
        return checked;
      },
    }));
    return (
      <View style={props.viewStyle}>
        <Text
          style={[
            RadioButtonStyles.text,
            RadioButtonStyles.label,
            props.labelStyle,
          ]}>
          {props.label}
        </Text>
        {props.options.map(option => (
          <View style={RadioButtonStyles.container} key={option}>
            <RadioButton
              {...props}
              value={option}
              status={checked === option ? 'checked' : 'unchecked'}
              onPress={() => onPressHandler(option)}
            />
            <TouchableWithoutFeedback onPress={() => onPressHandler(option)}>
              <Text style={RadioButtonStyles.text}>{option}</Text>
            </TouchableWithoutFeedback>
          </View>
        ))}
      </View>
    );
  },
);

const RadioButtonStyles = StyleSheet.create({
  container: {
    flexDirection: 'row',
    alignItems: 'center',
    justifyContent: 'flex-start',
  },
  text: {
    color: 'black',
  },
  label: {
    fontSize: 15,
    fontWeight: 'bold',
  },
});
